@@ -2,26 +2,41 @@
 # !/usr/bin/env python3
 ###########################import package###################
 import os
+import pathlib
+
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
+# https://blog.csdn.net/geek64581/article/details/106052164
+# matplotlib 中文字体解决方法
 import matplotlib as mpl
 import json
 import re
 import sys
+# 解析命令行的工具
 import argparse
 from typing import Union # python3 类型检查
+from typing import List, Dict, Tuple, Generator, Callable
 import logging
 import time
 from logging import Logger
 from logging.handlers import TimedRotatingFileHandler
 import click
 from pathlib import Path
+import datetime
+# 处理中国节假日的包
+from chinese_calendar import is_workday, is_holiday
+import unittest
+import gzip
+from multiprocessing import Pool, cpu_count
+import requests
+from bs4 import BeautifulSoup
 ###########################import package###################
 
 # 获取用户的家目录
 HOME = os.path.expanduser("~")
 _PROJECT_DIR = Path(__file__).parent.parent
+PROJECT_DIR = _PROJECT_DIR
 _PROJECT_LOG_DIR = _PROJECT_DIR / "log"
 # 将一个列表套列表的对象展开（展开嵌套列表）
 def flat(nums):
@@ -44,12 +59,14 @@ def change_en_to_cn_punctuation(string):
         new_string = new_string +"。"
     return new_string
 
-# write a simple data to a json file
+# 讲一个python对象写入json文件中
+# v1 to_json_file 20210201 废弃
 def to_json_file(obj_name, file_name):
     with open(file_name, 'w') as f:
         json.dump(obj_name, f, ensure_ascii=False, indent=4)
 
 class NpEncoder(json.JSONEncoder):
+
     def default(self, obj):
         if isinstance(obj, np.integer):
             return int(obj)
@@ -64,7 +81,7 @@ class NpEncoder(json.JSONEncoder):
 
 # write a complex data to a json file
 def obj_to_json(obj, file_path):
-    json_str = json.dumps(obj, indent=4, ensure_ascii=False, cls=NpEncoder)
+    json_str = json.dumps(obj, indent=4, cls=NpEncoder, ensure_ascii=False)
     with open(file_path, 'w') as json_file:
         json_file.write(json_str)
 
@@ -77,16 +94,18 @@ def json_file_to_obj(file_path):
 def mkdir(outdir):
     try:
         os.makedirs(outdir)
-    except Exception as e:
-        print("目录已存在：{}".format(outdir))
+        # os.mkdir(outdir)
+    except FileExistsError:
+        # print("{} exist...".format(outdir))
+        pass
 
 ###########################在bash中执行命令 start##########################
 # 运行没有返回值（不需要在执行后的返回值）的命令
 def run_command(command: str):
     if os.system(command) == 0 :
-        print('command run succeeded: {}'.format(command))
+        return True
     else:
-        print('command run failed: {}'.format(command))
+        return False
 
 # 运行具有返回值（在命令行输出的值）的命令
 def run_command_popen(command: str) -> str:
@@ -208,12 +227,11 @@ def timethis(func):
         start = time.time()
         r = func(*args, **kwargs)
         end = time.time()
-        print("上传用时：",end-start)
+        print("use time：",end-start)
         return r
     return wrapper
 #############################################
 
-# Todo：中文显示问题
 # 绘图函数区 start
 def draw_pie(labels=None, # 每份数据的标签
              quants=None, # 数据组成的列表
@@ -248,34 +266,37 @@ def draw_pie(labels=None, # 每份数据的标签
     plt.pie(quants, explode=expl, colors=colors, labels=labels, autopct='%1.1f%%',pctdistance=0.8, shadow=True)
     plt.title(title_name, bbox={'facecolor':'0.8', 'pad':5})
     plt.savefig(png_path)
-    plt.show()
+    # plt.show()
     plt.close()
 # 绘图函数区 end
 ##########################tag：数据分析#################################
 # 对table型文件的处理函数区 start
 # 将列表转化为以tab分割的字符串
 def list_to_line(column_list):
-    column_list = (str(item) for item in column_list)
+    column_list = (str(item).strip() for item in column_list)
     return '\t'.join(column_list)
 
 # 将以tab分割的字符串(一般是从文件中读取的一行）转化为列表
 def line_to_list(line):
     # return line.strip().split('\t')
-    return line.replace("\n", "").replace("\r", "").split("\t")
+    line_list = line.replace("\n", "").replace("\r", "").split("\t")
+    return [item.strip() if item.strip() else "null" for item in line_list]
 
 # 对dataframe按照列字段进行去重
 def drop_duplicates(df, colums_name=None):
     if colums_name is None:
-        return df.drop_duplicates()
+        return df.drop_duplicates(inplace=False)
     else:
         return df.drop_duplicates(subset=colums_name,keep='first',inplace=False)
 
 # 将dataframe对象写到文件中
-def df_to_tsv(df, file_name=None):
+def df_to_tsv(df: pd.DataFrame, file_name=None):
     if file_name is None:
         file_name = 'default.tsv'
     df.to_csv(file_name, sep='\t', header=True, index=False)
 
+def tsv_to_df(file_name: Union[str, pathlib.Path]) -> pd.DataFrame:
+    return pd.read_csv(file_name, sep="\t")
 # 将dataframe对象写到文件中
 def df_to_excel(df, file_name=None, sheet_name=None):
     if file_name is None:
@@ -319,7 +340,7 @@ def dfs_to_one_excel(df_list=None, # dataframe列表
 # 将一个列表形式的表格结构数据转化为dataframe数据结构
 def listTable_to_df(listTable, # 表格[[],]
                     header=None, # 表格的第一行为columns
-                    columns=None, # 表格的第一行为columns
+                    columns=None, # 指定表格的columns
                     ):
     if header==1:
         title_name = listTable[0]
@@ -336,7 +357,7 @@ def handle_data_by_row_df(df, # dataframe数据格式
                           handle_func # 行数据处理函数，接受一个series格式数据，并返回修改后的数据
                           ):
     series_list = []
-    for i, row in df.iterrows():
+    for _, row in df.iterrows():
         row = handle_func(row)
         series_list.append(row)
     df_out = pd.DataFrame(series_list)
@@ -385,8 +406,7 @@ def df_to_json_file(df:pd.DataFrame, json_file_path:str)->None:
     :param json_file_path: 待写入文件的路径
     :return:
     """
-    with open(json_file_path, "w") as f:
-        json.dump(df.to_dict(orient="index"), f, ensure_ascii=False, indent=4)
+    df.to_json(json_file_path, force_ascii=False, orient="records", indent=4)
 
 def json_file_to_df(json_file_path:str) ->pd.DataFrame:
     """
@@ -397,6 +417,46 @@ def json_file_to_df(json_file_path:str) ->pd.DataFrame:
     with open(json_file_path, "r") as f:
         return pd.DataFrame(json.loads(f.read())).T
 
+def df_to_dict(df: pd.DataFrame) -> dict:
+    """
+    从dataFrame生成字典，第一列为key，第二列为value
+    | key | value |
+    | --- | --- |
+    | bar | 123 |
+    | foo | 456 |
+    >>> df = pd.DataFrame([{"key": "bar", "value": 123}, {"key": "foo", "value": 456}])
+    >>> df
+       key  value
+    0  bar    123
+    1  foo    456
+    >>> df_to_dict(df)
+    {'bar': 123, 'foo': 456}
+    """
+    return df.set_index([df.columns[0]])[df.columns[1]].to_dict()
+
+def dict_to_df(dict: dict, columns=None) -> pd.DataFrame:
+    """
+    将key-value 字典形式的数据转换为dataFrame数据
+    >>> foo = { "bar":123 , "foo": 456}
+    >>> dict_to_df(foo, columns=["key", "value"])
+       key  value
+    0  bar    123
+    1  foo    456
+    """
+    res = []
+    if columns is None:
+        columns = ["key", "value"]
+    for key, value in dict.items():
+        temp_dict = {}
+        temp_dict[columns[0]] = key
+        temp_dict[columns[1]] = value
+        res.append(temp_dict)
+    return pd.DataFrame(res)
+
+# 将列表写到文件中
+def list_to_file(list_obj: list, title: str, file_path: Union[str, Path]):
+    df_to_tsv(pd.DataFrame(list_obj, columns=[title,]), file_path)
+
 # 按照索引横向合并表格
 def merge_two_df_by_left_index(df_left: pd.DataFrame, df_right: pd.DataFrame) -> pd.DataFrame:
     return pd.merge(df_left, df_right, right_index=True, left_index=True,)  # how='left'
@@ -439,11 +499,11 @@ def init_logger(logger_name, log_dir=_PROJECT_LOG_DIR):
 #     logger.exception("Faild to open sklearn.txt from logger.error")
 
 
-if __name__ == '__main__':
-    logger = init_logger("utils_slliu_test")
-    logger.error("test-error")
-    logger.info("test-info")
-    logger.warning("test-warn")
+# if __name__ == '__main__':
+#     logger = init_logger("utils_slliu_test")
+#     logger.error("test-error")
+#     logger.info("test-info")
+#     logger.warning("test-warn")
 
 
 
